"""CLI package for Oboyu.

This package provides the command-line interface for the Oboyu system.
"""

from oboyu.cli.main import run


def main() -> None:
    """Run the Oboyu CLI application."""
    run()
